package de.ozgcloud.bescheid;

import java.util.Collection;
import java.util.List;

import de.ozgcloud.bescheid.BescheidServiceGrpc.BescheidServiceImplBase;
import de.ozgcloud.bescheid.attacheditem.AttachedItem;
import de.ozgcloud.bescheid.attacheditem.AttachedItemMapper;
import de.ozgcloud.bescheid.attacheditem.AttachedItemService;
import de.ozgcloud.bescheid.vorgang.VorgangId;
import io.grpc.stub.StreamObserver;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
@RequiredArgsConstructor
class BescheidGrpcService extends BescheidServiceImplBase {

	private final AttachedItemService attachedItemService;
	private final AttachedItemMapper attachedItemMapper;
	private final BescheidService bescheidService;
	private final GrpcBescheidMapper grpcBescheidMapper;

	@Override
	public void getBescheidDraft(GrpcGetBescheidDraftRequest request, StreamObserver<GrpcGetBescheidDraftResponse> responseObserver) {
		var response = attachedItemService.findBescheidItem(request.getVorgangId())
				.map(this::buildGetBescheidDraftResponse)
				.orElseGet(GrpcGetBescheidDraftResponse::getDefaultInstance);

		responseObserver.onNext(response);
		responseObserver.onCompleted();

	}

	GrpcGetBescheidDraftResponse buildGetBescheidDraftResponse(AttachedItem bescheidItem) {
		return GrpcGetBescheidDraftResponse.newBuilder()
				.setBescheid(attachedItemMapper.toBescheid(bescheidItem))
				.build();
	}

	@Override
	public void getConfig(GrpcBescheidManagerConfigRequest request, StreamObserver<GrpcBescheidManagerConfigResponse> responseObserver) {
		responseObserver.onNext(grpcBescheidMapper.fromBescheidManagerConfig(bescheidService.getConfig()));
		responseObserver.onCompleted();
	}

	@Override
	public void getAll(GrpcGetAllBescheidRequest request, StreamObserver<GrpcGetAllBescheidResponse> responseObserver) {
		responseObserver.onNext(buildGetAllBescheidResponse(attachedItemService.findAllBescheid(VorgangId.from(request.getVorgangId()))));
		responseObserver.onCompleted();
	}

	GrpcGetAllBescheidResponse buildGetAllBescheidResponse(Collection<AttachedItem> bescheidItems) {
		return GrpcGetAllBescheidResponse.newBuilder().addAllBescheid(mapBescheid(bescheidItems)).build();
	}

	List<GrpcBescheid> mapBescheid(Collection<AttachedItem> bescheidItems) {
		return bescheidItems.stream().map(attachedItemMapper::toBescheid).toList();
	}
}
