package de.ozgcloud.bescheid.dummy;

import java.util.Optional;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import com.google.common.net.MediaType;

import de.ozgcloud.bescheid.Bescheid;
import de.ozgcloud.bescheid.BescheidRemoteService;
import de.ozgcloud.bescheid.BescheidRequest;
import de.ozgcloud.bescheid.vorgang.Vorgang;
import de.ozgcloud.common.binaryfile.TempFileUtils;

@Service
@ConditionalOnProperty("ozgcloud.feature.bescheid.enable-dummy-document-processor")
class DummyBescheidRemoteService implements BescheidRemoteService {

	private static final String DUMMY_BESCHEID_FILE_NAME = "dummy-bescheid.pdf";
	private static final String DUMMY_BESCHEID_CONTENT_TYPE = MediaType.PDF.toString();

	@Override
	public Bescheid create(BescheidRequest request, Vorgang vorgang) {
		var file = TempFileUtils.writeTmpFile(this.getClass().getClassLoader().getResourceAsStream(DUMMY_BESCHEID_FILE_NAME));

		return Bescheid.builder()
				.bescheidFile(file)
				.bescheidFileName(DUMMY_BESCHEID_FILE_NAME)
				.contentType(DUMMY_BESCHEID_CONTENT_TYPE)
				.nachrichtText(Optional.of("Dummy Bescheid"))
				.nachrichtSubject(Optional.of("Nachricht Subject"))
				.size(file.length())
				.createdBy(request.getCreateFor().getId())
				.build();
	}

}
