/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.files;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationOperation;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.stereotype.Repository;

import de.ozgcloud.vorgang.vorgang.Vorgang;

@Repository
class EingangFilesRepository {

	@Autowired
	private MongoTemplate mongoTemplate;

	private static final String DB_FIELDNAME_EINGANG_ID = "eingangs._id";
	private static final String DB_FIELDNAME_EINGANGS = "eingangs";
	private static final String DB_FIELDNAME_ATTACHMENTS = "attachments";
	private static final String DB_FIELDNAME_FILES = "files";
	private static final String DB_FIELDNAME_REPRESENTATIONS = "representations";

	private static final String DB_FIELDNAME_FILE_ID = "id";
	private static final String DB_FIELDNAME_FILE_NAME = "name";
	private static final String DB_FIELDNAME_FILE_SIZE = "size";
	private static final String DB_FIELDNAME_FILE_CONTENT_TYPE = "contentType";

	public List<OzgFile> findAttachmentsByEingangId(String eingangId) {

		List<AggregationOperation> operations = new ArrayList<>();

		operations.add(Aggregation.match(new Criteria(DB_FIELDNAME_EINGANG_ID).is(eingangId)));
		operations.addAll(buildFindAttachmentsAggregation());
		operations.add(Aggregation.project(DB_FIELDNAME_FILE_ID, DB_FIELDNAME_FILE_NAME, DB_FIELDNAME_FILE_SIZE, DB_FIELDNAME_FILE_CONTENT_TYPE));

		return mongoTemplate.aggregate(Aggregation.newAggregation(operations), Vorgang.COLLECTION_NAME, OzgFile.class).getMappedResults();
	}

	private List<AggregationOperation> buildFindAttachmentsAggregation() {

		return List.of(
				Aggregation.project(DB_FIELDNAME_EINGANGS),
				Aggregation.unwind(DB_FIELDNAME_EINGANGS),
				Aggregation.replaceRoot(DB_FIELDNAME_EINGANGS),
				Aggregation.project(DB_FIELDNAME_ATTACHMENTS),
				Aggregation.unwind(DB_FIELDNAME_ATTACHMENTS),
				Aggregation.replaceRoot(DB_FIELDNAME_ATTACHMENTS),
				Aggregation.project(DB_FIELDNAME_FILES),
				Aggregation.unwind(DB_FIELDNAME_FILES),
				Aggregation.replaceRoot(DB_FIELDNAME_FILES));
	}

	List<OzgFile> findRepresentationsByEingangId(String eingangId) {

		List<AggregationOperation> operations = new ArrayList<>();

		operations.add(Aggregation.match(new Criteria(DB_FIELDNAME_EINGANG_ID).is(eingangId)));
		operations.addAll(buildFindRepresentationsAggregation());
		operations.add(Aggregation.project(DB_FIELDNAME_FILE_ID, DB_FIELDNAME_FILE_NAME, DB_FIELDNAME_FILE_SIZE, DB_FIELDNAME_FILE_CONTENT_TYPE));

		return mongoTemplate.aggregate(Aggregation.newAggregation(operations), Vorgang.COLLECTION_NAME, OzgFile.class).getMappedResults();
	}

	private List<AggregationOperation> buildFindRepresentationsAggregation() {

		return List.of(
				Aggregation.project(DB_FIELDNAME_EINGANGS),
				Aggregation.unwind(DB_FIELDNAME_EINGANGS),
				Aggregation.replaceRoot(DB_FIELDNAME_EINGANGS),
				Aggregation.project(DB_FIELDNAME_REPRESENTATIONS),
				Aggregation.unwind(DB_FIELDNAME_REPRESENTATIONS),
				Aggregation.replaceRoot(DB_FIELDNAME_REPRESENTATIONS));
	}
}
