/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.files;

import java.io.ByteArrayInputStream;
import java.util.Date;

import org.bson.Document;
import org.bson.types.ObjectId;
import org.springframework.data.mongodb.gridfs.GridFsUpload;
import org.springframework.data.mongodb.util.BsonUtils;

import com.mongodb.client.gridfs.model.GridFSFile;

import de.ozgcloud.vorgang.callcontext.CallContextTestFactory;
import de.ozgcloud.vorgang.callcontext.UserTestFactory;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;

public class GridFsTestFactory {

	// TODO Konstante sinnvoll unterbringen
	public static final String COLLECTION_NAME = "fs.files";

	public static final ObjectId ID = new ObjectId();
	public static final Date UPLOAD_DATE = new Date();
	public static final Integer CHUNK_SIZE = 1;

	public static GridFSFile create() {
		return new GridFSFile(BsonUtils.simpleToBsonValue(ID), OzgFileTestFactory.NAME, OzgFileTestFactory.SIZE, CHUNK_SIZE, UPLOAD_DATE,
				createMetadata());
	}

	public static GridFsUpload<ObjectId> createUpload() {
		return createUploadBuilder(OzgFileTestFactory.CONTENT).build();
	}

	public static GridFsUpload<ObjectId> createUpload(byte[] content) {
		return createUploadBuilder(content)
				.id(ID)
				.build();
	}

	public static GridFsUpload.GridFsUploadBuilder<ObjectId> createUploadBuilder(byte[] content) {
		return GridFsUpload.fromStream(new ByteArrayInputStream(content))
				.metadata(createMetadata())
				.filename(createFilePath());
	}

	private static Document createMetadata() {
		Document metadata = new Document();
		metadata.append(BinaryFileRepository.VORGANG_ID, VorgangTestFactory.ID);
		metadata.append(BinaryFileRepository.CLIENT, CallContextTestFactory.CLIENT);
		metadata.append(BinaryFileRepository.FIELD_NAME, UploadedFilesReferenceTestFactory.NAME);
		metadata.append(BinaryFileRepository.CONTENT_TYPE, OzgFileTestFactory.CONTENT_TYPE);
		metadata.append(BinaryFileRepository.FILE_NAME, OzgFileTestFactory.NAME);
		metadata.append(BinaryFileRepository.CREATED_BY, UserTestFactory.ID);

		return metadata;
	}

	private static String createFilePath() {
		return String.format("%s/%s/%s/%s", VorgangTestFactory.ID, CallContextTestFactory.CLIENT,
				UploadedFilesReferenceTestFactory.NAME, OzgFileTestFactory.NAME);
	}
}