/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.files;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Random;

import de.ozgcloud.vorgang.vorgang.IncomingFileTestFactory;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;

public class OzgFileTestFactory {

	public static final FileId ID = FileId.createNew();
	public static final String NAME = "Filename1.pdf";
	public static final String CONTENT_TYPE = IncomingFileTestFactory.CONTENT_TYPE_STR;

	public static final byte[] CONTENT = "juhu, ein bild".getBytes();
	public static final long SIZE = CONTENT.length;
	public static final InputStream CONTENT_STREAM = new ByteArrayInputStream(CONTENT);

	public static OzgFile create() {
		return createBuilder().build();
	}

	public static OzgFile.OzgFileBuilder createBuilder() {
		return OzgFile.builder()
				.id(ID)
				.vorgangId(VorgangTestFactory.ID)
				.name(NAME)
				.contentType(CONTENT_TYPE)
				.size(SIZE);
	}

	public static byte[] createContentInMiB(int sizeInMiB) {
		return createContentInByte(sizeInMiB * 1_000_000);
	}

	public static byte[] createContentInByte(int fileSize) {
		Random r = new Random();
		byte[] chunk = new byte[fileSize];
		for (int i = 0; i < chunk.length; i++) {
			chunk[i] = (byte) (r.nextInt('z' - 'a') + 'a');
		}
		return chunk;
	}
}
