/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.vorgang.redirect;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.vorgang.forwarding.GrpcForwarding;
import de.ozgcloud.vorgang.forwarding.GrpcForwardingsResponse;
import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;
import io.grpc.stub.StreamObserver;

class ForwardingGrpcServiceTest {

	@InjectMocks // NOSONAR
	private ForwardingGrpcService grpcService;
	@Mock
	private ForwardingService service;
	@Mock
	private ForwardingMapper mapper;

	@Mock
	private StreamObserver<GrpcForwardingsResponse> responseObserver;

	@Nested
	class TestFindForwardings {

		private Forwarding forwarding = ForwardingTestFactory.create();
		private GrpcForwarding grpcForwarding = GrpcForwarding.newBuilder().build();

		@Captor // NOSONAR
		private ArgumentCaptor<GrpcForwardingsResponse> responseCaptor;

		@BeforeEach
		void init() {
			when(service.findForwardings(anyString())).thenReturn(Stream.of(forwarding));
			when(mapper.toGrpc(any())).thenReturn(GrpcForwarding.newBuilder().build());
		}

		@Test
		void shouldCallService() {
			grpcService.findForwardings(GrpcFindForwardingsRequestTestFactory.create(), responseObserver);

			verify(service).findForwardings(VorgangTestFactory.ID);
		}

		@Test
		void shouldCallMapper() {
			grpcService.findForwardings(GrpcFindForwardingsRequestTestFactory.create(), responseObserver);

			verify(mapper).toGrpc(forwarding);
		}

		@Test
		void shouldWriteGrpcForwarding() {
			grpcService.findForwardings(GrpcFindForwardingsRequestTestFactory.create(), responseObserver);

			verify(responseObserver).onNext(responseCaptor.capture());
			assertThat(responseCaptor.getValue().getForwardingsList()).hasSize(1).contains(grpcForwarding);
		}

		@Test
		void shouldCompleteResponse() {
			grpcService.findForwardings(GrpcFindForwardingsRequestTestFactory.create(), responseObserver);

			verify(responseObserver).onCompleted();
		}
	}

}
